#!/bin/bash

start_time=`date +%s`

[ ! $# -eq 5 ] && { echo "Usage: $0 id short1.fastq.gz short2.fastq.gz long.bam reference.fna.gz"; exit 1; }

id=$1
short1=$2
short2=$3
long=$4
ref=$5
cores=16

[[ ! -e $short1 ]] && { echo $short1 does not exists; exit 1; }
[[ ! -e $short2 ]] && { echo $short2 does not exists; exit 1; }
[[ ! -e $long ]] && { echo $long does not exists; exit 1; }

# short read quality filtering
#export PATH=$PATH:/work_zfs2/sukem066/software/fastq_illumina_filter/fastq_illumina_filter-0.1/
parallel gunzip -c {} '|' fastq_illumina_filter --keep N -vv -o filtered{/.} ::: $short1 $short2

# filter bad quality reads
parallel prinseq-lite.pl  -fastq filtered{/.} -min_len 20 -ns_max_n 8 -min_qual_mean 15 -trim_qual_left 12 -trim_qual_right 12 -out_format 3 -out_good good{/.}  -out_bad bad{/.} ::: $short1 $short2
in1=$(echo good`basename "$short1"` | sed 's/.gz$//')
in2=$(echo good`basename "$short2"` | sed 's/.gz$//')
out1=$(echo `basename "$short1"` | sed 's/.fastq/_filtered.fastq/')
out2=$(echo `basename "$short2"` | sed 's/.fastq/_filtered.fastq/')
repair.sh in=$in1.fastq in2=$in2.fastq out=$out1 out2=$out2 # bbmap
rm $in1.fastq $in2.fastq `echo $in1.fastq | sed 's/^good/bad/'` `echo $in2.fastq | sed 's/^good/bad/'` `echo $in1 | sed 's/^good/filtered/'` `echo $in2 | sed 's/^good/filtered/'`
short1F=$out1
short2F=$out2


####################
# 1) hybrid spades
####################
spades.py -t $cores -k 21,33,55,77,99,127 --careful -1 $short1F -2 $short2F --pacbio $long -o ${id}_1a >/dev/null
cat ${id}_1a/spades.log | grep -E "Average coverage|WARN|ERROR"
spades.py -t $cores -k 21,33,55,77,99 --careful -1 $short1F -2 $short2F --pacbio $long -o ${id}_1b >/dev/null # lower kmer

N50_1a=$(abyss-fac ${id}_1a/scaffolds.fasta | grep ${id}_1a | cut -f6)
N50_1b=$(abyss-fac ${id}_1b/scaffolds.fasta | grep ${id}_1b | cut -f6)
if (( $N50_1a > $N50_1b )); then
    spades_kmer="21,33,55,77,99,127"
else
    spades_kmer="21,33,55,77,99"
fi


# estimate genome size (needed for canu)
est_genome_size=$(cat ${id}_1a/scaffolds.fasta | grep -Eo "length_[0-9]+" | grep -Eo "[0-9]+" | tr '\n' '+' | rev | cut -c2- | rev | bc)

# correct long reads with short reads (lordec)
bam2fasta -o ${id}_long $long
lordec-correct -T $cores -2 $short1F,$short2F -k 19 -s 3 -i ${id}_long.fasta.gz -o ${id}_lordec.fasta >/dev/null
bam2fastq -o ${id}_long $long

# correct long reads with short reads (proovread)
#export PATH=$PATH:/work_zfs2/sukem066/software/proovread/bin/
#gunzip -k $short1F $short2F ${id}_long.fasta.gz # does only work for gunzip >=1.6
gunzip -c $short1F > ${short1F%.*}
gunzip -c $short2F > ${short2F%.*}
gunzip -c ${id}_long.fasta.gz > ${id}_long.fasta
proovread -t $cores -p ${id}_proovread -l ${id}_long.fasta -s ${short1F%.*} -s ${short2F%.*} >/dev/null
mv ${id}_proovread/${id}_proovread.trimmed.fa ${id}_proovread.fasta
rm -rf ${id}_proovread

#canu correction
canu -correct useGrid=false maxThreads=$cores -p $id -d canu_corrected genomeSize=$est_genome_size -pacbio-raw ${id}_long.fasta.gz >/dev/null
mv canu_corrected/${id}.correctedReads.fasta.gz ./${id}_canu.fasta.gz
rm -rf canu_corrected


##########
# 2) canu
#########
canu useGrid=false maxThreads=$cores -p $id -d ${id}_2a genomeSize=$est_genome_size -pacbio-raw ${id}_long.fasta.gz >/dev/null
canu useGrid=false maxThreads=$cores -p $id -d ${id}_2b genomeSize=$est_genome_size -pacbio-corrected ${id}_lordec.fasta >/dev/null
canu useGrid=false maxThreads=$cores -p $id -d ${id}_2c  genomeSize=$est_genome_size -pacbio-corrected ${id}_proovread.fasta  >/dev/null
#canu useGrid=false maxThreads=$cores -trim-assemble -p $id -d ${id}_2d  genomeSize=$est_genome_size -pacbio-corrected ${id}_proovread.fasta # no difference
#canu useGrid=false maxThreads=$cores -p $id -d ${id}_2e genomeSize=$est_genome_size -pacbio-raw ${id}_proovread.fasta # worse results


###############################
# 3) spades with canu corrected as trusted contigs
###############################
spades.py -t $cores -k $spades_kmer --careful -1 $short1F -2 $short2F --pacbio $long --trusted-contigs ${id}_2a/${id}.contigs.fasta -o ${id}_3a >/dev/null
spades.py -t $cores -k $spades_kmer --careful -1 $short1F -2 $short2F --pacbio $long --trusted-contigs ${id}_2b/${id}.contigs.fasta -o ${id}_3b >/dev/null
spades.py -t $cores -k $spades_kmer --careful -1 $short1F -2 $short2F --pacbio $long --trusted-contigs ${id}_2c/${id}.contigs.fasta -o ${id}_3c >/dev/null


################################
# 4) spades hybrid with corrected
################################
spades.py -t $cores -k $spades_kmer --careful -1 $short1F -2 $short2F --pacbio ${id}_canu.fasta.gz -o ${id}_4a >/dev/null
spades.py -t $cores -k $spades_kmer --careful -1 $short1F -2 $short2F --pacbio ${id}_lordec.fasta -o ${id}_4b >/dev/null
spades.py -t $cores -k $spades_kmer --careful -1 $short1F -2 $short2F --pacbio ${id}_proovread.fasta -o ${id}_4c >/dev/null


#############
# 5) masurca hybrid (and with canu assemblies as reference)
############
mkdir ${id}_5a && cd ${id}_5a
echo -e "
PARAMETERS 
USE_LINKING_MATES=0
LIMIT_JUMP_COVERAGE = 60
CA_PARAMETERS = cgwErrorRate=0.25
NUM_THREADS=$cores
JF_SIZE=100000000
END

DATA
PE= p1 500 50 ../$short1F ../$short2F
PACBIO=../${id}_long.fasta.gz
END
" > masurca.conf
masurca masurca.conf
./assemble.sh
find . -type f ! -regex ".*assembly.fasta\|.*final.genome.scf.fasta\|.*\.log" -exec rm -f {} + # save space
cd ..


cat ${id}_2*/${id}.contigs.fasta > ${id}_canu_all.fasta # use all canu assemblies as reference
mkdir ${id}_5b && cd ${id}_5b
echo -e "
PARAMETERS 
USE_LINKING_MATES=0
LIMIT_JUMP_COVERAGE = 60
CA_PARAMETERS = cgwErrorRate=0.25
NUM_THREADS=$cores
JF_SIZE=100000000
END

DATA
PE= p1 500 50 ../$short1F ../$short2F
PACBIO=../${id}_long.fasta.gz
REFERENCE=../${id}_canu_all.fasta
END
" > masurca.conf
masurca masurca.conf
./assemble.sh
find . -type f ! -regex ".*assembly.fasta\|.*final.genome.scf.fasta\|.*\.log" -exec rm -f {} + # save space
cd ..


#############
# 6) masurca hybrid wth corrected 
############
mkdir ${id}_6a && cd ${id}_6a
echo -e "
PARAMETERS 
USE_LINKING_MATES=0
LIMIT_JUMP_COVERAGE = 60
CA_PARAMETERS = cgwErrorRate=0.25
NUM_THREADS=$cores
JF_SIZE=100000000
END

DATA
PE= p1 500 50 ../$short1F ../$short2F
PACBIO=../${id}_canu.fasta.gz
END
" > masurca.conf
masurca masurca.conf
./assemble.sh
find . -type f ! -regex ".*assembly.fasta\|.*final.genome.scf.fasta\|.*\.log" -exec rm -f {} + # save space
cd ..


mkdir ${id}_6b && cd ${id}_6b
echo -e "
PARAMETERS 
USE_LINKING_MATES=0
LIMIT_JUMP_COVERAGE = 60
CA_PARAMETERS = cgwErrorRate=0.25
NUM_THREADS=$cores
JF_SIZE=100000000
END

DATA
PE= p1 500 50 ../$short1F ../$short2F
PACBIO=../${id}_lordec.fasta
END
" > masurca.conf
masurca masurca.conf
./assemble.sh
find . -type f ! -regex ".*assembly.fasta\|.*final.genome.scf.fasta\|.*\.log" -exec rm -f {} + # save space
cd ..


mkdir ${id}_6c && cd ${id}_6c
echo -e "
PARAMETERS 
USE_LINKING_MATES=0
LIMIT_JUMP_COVERAGE = 60
CA_PARAMETERS = cgwErrorRate=0.25
NUM_THREADS=$cores
JF_SIZE=100000000
END

DATA
PE= p1 500 50 ../$short1F ../$short2F
PACBIO=../${id}_proovread.fasta
END
" > masurca.conf
masurca masurca.conf
./assemble.sh
find . -type f ! -regex ".*assembly.fasta\|.*final.genome.scf.fasta\|.*\.log" -exec rm -f {} + # save space
cd ..



###########
# Unicycler
###########
unicycler --verbosity 0 -t $cores --keep 0 -1 $short1F -2 $short2F -l ${id}_long.fasta.gz -o ${id}_7a


#######
# EVAL
#######
assemblies=$(ls ${id}_1*/scaffolds.fasta ${id}_2*/${id}.contigs.fasta ${id}_3*/scaffolds.fasta ${id}_4*/scaffolds.fasta ${id}_5*/CA.mr.41.15.17.0.029/final.genome.scf.fasta ${id}_5*/flye.mr.41.17.20.0.05/assembly.fasta ${id}_6*/CA.mr.41.15.17.0.029/final.genome.scf.fasta ${id}_7*/assembly.fasta)


# quast
function join_by { local IFS="$1"; shift; echo "$*"; }
labels=$(join_by , `echo $assemblies | grep -Eo "${id}_[0-9][a-z]"`)
#export PATH=$PATH:/work_zfs2/sukem066/software/quast/quast-5.0.2
#quast.py --threads $cores --silent --rna-finding -b -l $labels -R $ref --fragmented --glimmer -1 $short1F -2 $short2F --pacbio ${id}_long.fastq.gz $assemblies -o ${id}_quast --space-efficient


#abyss (short statistics)
abyss-fac $assemblies


###########
# cleaning
###########
#rm $short1F $short2F ${short1F%.*} ${short2F%.*}
#rm ${id}_long.fasta.gz ${id}_long.fasta ${id}_long.fastq.gz
#rm ${id}_canu.fasta.gz ${id}_proovread.fasta ${id}_lordec.fasta
#rm ${id}_canu_all.fasta

echo "variables used:
id=$1
short1=$2
short2=$3
long=$4
short1F=$short1F
short2F=$short2F
ref=$5
cores=$cores
spades_kmer=$spades_kmer
"
end_time=`date +%s`
echo Running time: `expr $end_time - $start_time` s.
